-- AnnoyingBuffReminder 2.14, by Wikwocket (wikwocket@gmail.com)
-- 
-- Displays an annoying reminder to use armor spells, auras, shouts, etc, and Victory Rush.

-- pre-release version history:
-- 1.0 initial version
-- 1.1 added commanding shout, warlock and mage armors, victory rush
-- 1.1b updated victory rush to not show if on cooldown
-- 1.2 do not show armor/shout reminder if player is resting, flying, mounted, or not in control of character
-- 1.3 added shaman shields
-- 1.4 initial public release; changed victory rush modifier text color

local VERSION = "2.14";

-- Print: helper function to send message to default chat frame
local function Print(text)
	if (DEFAULT_CHAT_FRAME) then
		DEFAULT_CHAT_FRAME:AddMessage(text);
	end
end

local className = nil;
local inBG = false;
local inCombat = false;
local battleShoutOnly = false;
local commandingShoutOnly = false;

-- OnLoad: register events, slash commands, and hooks, print welcome message
function AnnoyingBuffReminderOnLoad()
	AnnoyingBuffReminderHoverFrame:Hide();
	_, className = UnitClass("player");
	SlashCmdList["ANNOYINGBUFFREMINDER"] = AnnoyingBuffReminderSlashCommand;
	SLASH_ANNOYINGBUFFREMINDER1 = "/annoyingbuffreminder";
	SLASH_ANNOYINGBUFFREMINDER2 = "/abr";
	AnnoyingBuffReminderHoverFrame:RegisterEvent("VARIABLES_LOADED");
	if (className == "WARRIOR") then
		Print("AnnoyingBuffReminder " .. VERSION .. " enabled, using WARRIOR reminders");
		AnnoyingBuffReminderHoverFrame:RegisterEvent("ZONE_CHANGED_NEW_AREA");
		--AnnoyingBuffReminderHoverFrame:RegisterEvent("UNIT_AURA");
		AnnoyingBuffReminderHoverFrame:RegisterEvent("UNIT_RAGE");
		--AnnoyingBuffReminderHoverFrame:RegisterEvent("ACTIONBAR_SLOT_CHANGED");
		--AnnoyingBuffReminderHoverFrame:RegisterEvent("PLAYER_ENTERING_WORLD");
		AnnoyingBuffReminderHoverFrame:RegisterEvent("ACTIONBAR_UPDATE_USABLE");
		AnnoyingBuffReminderHoverFrame:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN");
	elseif (className == "PRIEST" or className == "WARLOCK" or className == "MAGE" or className == "SHAMAN"
			or className == "HUNTER" or className == "PALADIN" or className == "DRUID" or className == "ROGUE"
			or className == "DEATHKNIGHT") then
		Print("AnnoyingBuffReminder " .. VERSION .. " enabled, using " .. className .. " reminders");
		AnnoyingBuffReminderHoverFrame:RegisterEvent("ZONE_CHANGED_NEW_AREA");
		AnnoyingBuffReminderHoverFrame:RegisterEvent("UNIT_AURA");
		AnnoyingBuffReminderHoverFrame:RegisterEvent("UNIT_MANA");
		-- optional in-combat check
		-- seals/sacred shield/demonic sacrifice/spellstone/rogue reminders only show in combat
		if (className == "PALADIN" or className == "WARLOCK"
			or className == "ROGUE" or className == "DEATHKNIGHT") then
			AnnoyingBuffReminderHoverFrame:RegisterEvent("PLAYER_REGEN_DISABLED");
			AnnoyingBuffReminderHoverFrame:RegisterEvent("PLAYER_REGEN_ENABLED");
		end
		-- additional energy type/misc checks
		if (className == "DRUID") then -- for druid forms
			AnnoyingBuffReminderHoverFrame:RegisterEvent("UNIT_RAGE");
			AnnoyingBuffReminderHoverFrame:RegisterEvent("UNIT_ENERGY");
		elseif (className == "ROGUE") then -- for rogue energy ticks
			AnnoyingBuffReminderHoverFrame:RegisterEvent("UNIT_ENERGY");
		elseif (className == "DEATHKNIGHT") then -- for death knight runic power ticks, and finding bone shield
			AnnoyingBuffReminderHoverFrame:RegisterEvent("UNIT_RUNIC_POWER");
		end
	end
end

-- SlashCommand: handle slash commands
function AnnoyingBuffReminderSlashCommand(command)
	-- prep temp info to help handle basic slash commands
	local slashCommandInfo = {
		["Demonic Sacrifice"] = { "demonsacOn", "ds", "demon", "demonsac", "sacrifice", "demonicsacrifice" },
		["Detect Invisibility"] = { "detectOn", "di", "detect", "invis", "detectinvis", "detectinvisibility" },
		["Soul Link"] = { "linkOn", "sl", "soul", "link", "soullink" },
		["Thorns"] = { "thornsOn", "t", "thorn", "thorns" },
		["Omen of Clarity"] = { "omenOn", "ooc", "omen", "clarity", "omenofclarity" },
		["Trueshot Aura"] = { "trueshotOn", "tsa", "ts", "trueshot", "trueshotaura" },
		["Hunger for Blood"] = { "hungerOn", "hfb", "hunger", "blood", "hungerblood", "hungerforblood" },
		["Divine Spirit"] = { "spiritOn", "divine", "spirit", "divinespirit" },
		["Shadow Protection"] = { "shadowOn", "sp", "shadow", "shadprot", "shadowprot", "shadowprotection" },
		["Crusader Aura"] = { "crusaderOn", "ca", "crusader", "aura", "crusaderaura" },
		["Righteous Fury"] = { "righteousFuryOn", "rf", "righteous", "fury", "righteousfury" },
		["Sacred Shield"] = { "sacredShieldOn", "sacred", "sacredshield" },
		["a Seal"] = { "sealOn", "s", "seal", "seals" },
		["a Firestone/Spellstone enchant"] = { "stoneOn", "stone", "firestone", "spellstone" },
		["Bone Shield"] = { "boneShieldOn", "bone", "shield", "boneshield" },
		["Horn of Winter"] = { "hornOn", "how", "horn", "winter", "hornofwinter" },
		["Unholy Blight"] = { "blightOn", "ub", "unholy", "blight", "unholyblight" },
	}
	
	-- search for and process basic commands, setting variable and reporting
	local i;
	for buffName, buffInfo in pairs(slashCommandInfo) do
		for i = 2, table.getn(buffInfo), 1 do
			if (command == buffInfo[i]) then
				AnnoyingBuffReminderVariables[buffInfo[1]] = not AnnoyingBuffReminderVariables[buffInfo[1]];
				if (AnnoyingBuffReminderVariables[buffInfo[1]]) then
					Print("AnnoyingBuffReminder: You will now be reminded to cast " .. buffName .. ".");
				else
					Print("AnnoyingBuffReminder: You will no longer be reminded to cast " .. buffName .. ".");
				end
				return;
			end
		end
	end
	
	-- handle special cases:
	-- battle shout
	if (command == "battle" or command == "battleshout" or command == "bs") then
		battleShoutOnly = not battleShoutOnly;
		if (battleShoutOnly) then
			commandingShoutOnly = false;
			Print("AnnoyingBuffReminder: You will now only be reminded to cast Battle Shout.");
		else
			Print("AnnoyingBuffReminder: You will now be reminded to cast either Battle Shout or Commanding Shout.");
		end
	-- commanding shout
	elseif (command == "command" or command == "commanding" or command == "commandingshout" or command == "cs") then
		commandingShoutOnly = not commandingShoutOnly;
		if (commandingShoutOnly) then
			battleShoutOnly = false;
			Print("AnnoyingBuffReminder: You will now only be reminded to cast Commanding Shout.");
		else
			Print("AnnoyingBuffReminder: You will now be reminded to cast either Battle Shout or Commanding Shout.");
		end
	-- any shout
	elseif (command == "any" or command == "anyshout" or command == "shout") then
		battleShoutOnly = false;
		commandingShoutOnly = false;
		Print("AnnoyingBuffReminder: You will now only be reminded when you don't have any shout active.");
	-- amp magic
	elseif (command == "amp" or command == "amplify" or command == "ampmagic" or command == "amplifymagic") then
		AnnoyingBuffReminderVariables.ampMagicOn = not AnnoyingBuffReminderVariables.ampMagicOn;
		if (AnnoyingBuffReminderVariables.ampMagicOn) then
			AnnoyingBuffReminderVariables.dampMagicOn = false;
			Print("AnnoyingBuffReminder: You will now be reminded to cast Amplify Magic.");
		else
			Print("AnnoyingBuffReminder: You will no longer be reminded to cast Amplify Magic.");
		end
	-- damp magic
	elseif (command == "damp" or command == "dampen" or command == "dampmagic" or command == "dampenmagic") then
		AnnoyingBuffReminderVariables.dampMagicOn = not AnnoyingBuffReminderVariables.dampMagicOn;
		if (AnnoyingBuffReminderVariables.dampMagicOn) then
			AnnoyingBuffReminderVariables.ampMagicOn = false;
			Print("AnnoyingBuffReminder: You will now be reminded to cast Dampen Magic.");
		else
			Print("AnnoyingBuffReminder: You will no longer be reminded to cast Dampen Magic.");
		end
	-- shaman shield (uses reversed logic so it can default to reminders being on)
	elseif (command == "shield" or command == "shaman" or command == "shieldspell") then
		AnnoyingBuffReminderVariables.shamanShieldOff = not AnnoyingBuffReminderVariables.shamanShieldOff;
		if (AnnoyingBuffReminderVariables.shamanShieldOff) then
			Print("AnnoyingBuffReminder: You will no longer be reminded to cast a Shaman shield spell.");
		else
			Print("AnnoyingBuffReminder: You will now be reminded to cast a Shaman shield spell.");
		end
	-- victory rush (uses reversed logic so it can default to reminders being on)
	elseif (command == "vr" or command == "victory" or command == "rush" or command == "victoryrush") then
		AnnoyingBuffReminderVariables.victoryRushOff = not AnnoyingBuffReminderVariables.victoryRushOff;
		if (AnnoyingBuffReminderVariables.victoryRushOff) then
			Print("AnnoyingBuffReminder: You will no longer be reminded to cast Victory Rush.");
		else
			Print("AnnoyingBuffReminder: You will now be reminded to cast Victory Rush.");
		end
	-- reminders in pvp
	elseif (command == "pvp" or command == "bg" or command == "battleground" or command == "arena") then
		AnnoyingBuffReminderVariables.remindInBG = not AnnoyingBuffReminderVariables.remindInBG;
		if (AnnoyingBuffReminderVariables.remindInBG) then
			Print("AnnoyingBuffReminder: You will now be reminded while in battlegrounds and arenas.");
		else
			Print("AnnoyingBuffReminder: You will no longer be reminded while in battlegrounds and arenas.");
		end
	-- lock
	elseif (command == "lock") then
		AnnoyingBuffReminderVariables.lock = true;
		AnnoyingBuffReminderHoverFrame:EnableMouse(false);
		Print("AnnoyingBuffReminder: Window locked. You can now click through the reminder window.");
	-- unlock frame
	elseif (command == "unlock") then
		AnnoyingBuffReminderVariables.lock = false;
		AnnoyingBuffReminderHoverFrame:EnableMouse(true);
		Print("AnnoyingBuffReminder: Window unlocked. You can now move the reminder window by clicking and dragging.");
	-- re-center frame
	elseif (command == "reset" or command == "center") then
		AnnoyingBuffReminderHoverFrame:ClearAllPoints();
		AnnoyingBuffReminderHoverFrame:SetPoint("CENTER", UIParent, "CENTER", 0, 0);
		Print("AnnoyingBuffReminder: Window position reset to center of screen.");
	-- toggle spell icon display
	elseif (command == "icon" or command == "icons") then
		AnnoyingBuffReminderVariables.dontShowIcons = not AnnoyingBuffReminderVariables.dontShowIcons;
		if (AnnoyingBuffReminderVariables.dontShowIcons) then
			Print("AnnoyingBuffReminder: Spell icons will no longer be shown.");
		else
			Print("AnnoyingBuffReminder: Spell icons will now be shown with spell names.");
		end
	-- default: print usage menu
	else
		Print("AnnoyingBuffReminder " .. VERSION);
		Print("Displays an annoying reminder to use armor spells, auras, shouts, etc, and Victory Rush.");
		Print("  /abr pvp||bg - Reminders in battlegrounds/arenas");
		Print("  /abr icons - Toggle display of spell icons");
		Print("  /abr lock||unlock - Toggle window locking and click-through");
		Print("  /abr center - Reset window position to center of screen");
		Print("Most reminders are automatic, but some are optional.");
		Print("The following may be toggled for your class, via /abr <spell>:");
		if (className == "DEATHKNIGHT") then Print("  Death Knights: bone||shield||boneshield||horn||winter||how||unholy||blight||ub"); end
		if (className == "DRUID")       then Print("  Druids: thorns||omen||clarity||ooc"); end
		if (className == "HUNTER")      then Print("  Hunters: tsa||trueshot"); end
		if (className == "MAGE")        then Print("  Mages: amp||amplify||damp||dampen"); end
		if (className == "PALADIN")     then Print("  Paladins: fury||righteous||righteousfury||rf||seal||crusader||ca||sacred||sacredshield"); end
		if (className == "PRIEST")      then Print("  Priests: divine||spirit||shadow||shadowprot"); end
		if (className == "ROGUE")       then Print("  Rogues: hungerforblood||hunger||hfb"); end
		if (className == "SHAMAN")      then Print("  Shamans: shield||shieldspell"); end
		if (className == "WARLOCK")     then Print("  Warlocks: soul||link||soullink||detect||invis||demon||sacrifice||stone"); end
		if (className == "WARRIOR")     then Print("  Warriors: battle||bs||commanding||cs||anyshout||vr||victory||rush"); end
	end
end

-- OnEvent:
function AnnoyingBuffReminderOnEvent(self, event, ...)
	local arg1 = select(1, ...);
	-- VARIABLES_LOADED: initialize
	if (event == "VARIABLES_LOADED") then
		-- load (or create) saved variables
		if (not AnnoyingBuffReminderVariables) then
			AnnoyingBuffReminderVariables = {};
			AnnoyingBuffReminderVariables.lock = false;
			-- let other vars init to nil/false
		end
		-- lock or unlock
		AnnoyingBuffReminderHoverFrame:EnableMouse(not AnnoyingBuffReminderVariables.lock);
		return;
	end
	
	-- ZONE_CHANGED_NEW_AREA: check for in a BG/arena
	if (event == "ZONE_CHANGED_NEW_AREA") then
		local inInstance, instanceType = IsInInstance(); -- note: IsInInstance()'s return values are same in all locales
		if (inInstance and (instanceType == "pvp" or instanceType == "arena")) then
			inBG = true;
		else
			inBG = false;
		end
		return;
	end
	
	-- PLAYER_REGEN_DISABLED: set combat flag
	if (event == "PLAYER_REGEN_DISABLED") then
		inCombat = true;
		return;
	end
	
	-- PLAYER_REGEN_ENABLED: clear combat flag
	if (event == "PLAYER_REGEN_ENABLED") then
		inCombat = false;
		return;
	end
	
	-- abort if inappropriate to notify player now
	if (UnitLevel("player") < 15 or (inBG and not AnnoyingBuffReminderVariables.remindInBG)
		or IsFlying() or (IsMounted() and className ~= "PALADIN") or UnitOnTaxi("player") or IsResting() or not HasFullControl()) then
			AnnoyingBuffReminderHoverFrame:Hide();
			return;
	end
	
	-- vars for whether ABR should be shown, used in code blocks below
	local showABR = false;
	local textABR = "";
	local spellABR = ""; -- used for determining spell icon texture
	local buffName;
	local casterID; -- note: only used for classes for whom all of the buffs we check are self-cast only
	
	-- ACTIONBAR_UPDATE_USABLE / ACTIONBAR_UPDATE_COOLDOWN / UNIT_RAGE from player: check for Victory Rush, shouts
	if (event == "ACTIONBAR_UPDATE_USABLE" or event == "ACTIONBAR_UPDATE_COOLDOWN" or (event == "UNIT_RAGE" and arg1 == "player" and className == "WARRIOR")) then
		if ((not AnnoyingBuffReminderVariables.victoryRushOff) and IsUsableSpell(ABR_LOCALE.victoryRush) and GetSpellCooldown(ABR_LOCALE.victoryRush) == 0) then
			showABR = true;
			textABR = "|cff00ff00Victory Rush!|r";
			spellABR = ABR_LOCALE.victoryRush;
		elseif (UnitPower("player", SPELL_POWER_RAGE) >= 10) then
			local needBuff = true;
			for i = 1, 40 do
				buffName = UnitAura("player", i, "HELPFUL");
				if (buffName) then
					if ((buffName == ABR_LOCALE.battleShout and not commandingShoutOnly)
						or (buffName == ABR_LOCALE.commandingShout and not battleShoutOnly)) then
							needBuff = false;
					end
				end
			end
			if (needBuff) then
				showABR = true;
				textABR = "Cast a Shout!";
				spellABR = ABR_LOCALE.battleShout;
			end
		end
		
		-- display or hide ABR as needed
		if (showABR) then
			AnnoyingBuffReminderText:SetText(textABR);
			AnnoyingBuffReminderHoverFrame:Show();
			if (AnnoyingBuffReminderVariables.dontShowIcons) then
				AnnoyingBuffReminderSpellButton1Icon:Hide();
				AnnoyingBuffReminderSpellButton2Icon:Hide();
				AnnoyingBuffReminderHoverFrame:SetWidth(200);
			else
				local texture = select(3, GetSpellInfo(spellABR));
				AnnoyingBuffReminderSpellButton1Icon:SetTexture(texture);
				AnnoyingBuffReminderSpellButton2Icon:SetTexture(texture);
				AnnoyingBuffReminderSpellButton1Icon:Show();
				AnnoyingBuffReminderSpellButton2Icon:Show();
				AnnoyingBuffReminderHoverFrame:SetWidth(265);
			end
		else
			AnnoyingBuffReminderHoverFrame:Hide();
		end
		return;	
	end
	
	-- UNIT_MANA/RAGE/ENERGY/RUNIC POWER from player (except warriors) / UNIT_AURA from player: check for appropriate armor spells/etc
	if ((event == "UNIT_MANA" or event == "UNIT_AURA" or event == "UNIT_ENERGY" or event == "UNIT_RUNIC_POWER"
	or (event == "UNIT_RAGE" and className == "DRUID")) and arg1 == "player") then
		if (className == "PRIEST") then
			-- inner fire, pw:fort, spirit, shadow
			local needIF = true;
			local needPWF = true;
			local needSpirit = true;
			local needShadow = true;
			for i = 1, 40 do
				buffName = UnitAura("player", i, "HELPFUL");
				--Print("buff " .. i .. " is " .. tostring(buffName) .. "/" .. tostring(casterID));
				if (buffName) then
					if (buffName == ABR_LOCALE.innerFire) then
						needIF = false;
					elseif (buffName == ABR_LOCALE.powerFortitude
						or buffName == ABR_LOCALE.prayerFortitude
						or buffName == ABR_LOCALE.songOfVictory) then
							needPWF = false;
					elseif (buffName == ABR_LOCALE.divineSpirit or buffName == ABR_LOCALE.prayerSpirit) then
						needSpirit = false;
					elseif (buffName == ABR_LOCALE.shadowProtection or buffName == ABR_LOCALE.prayerShadowProtection) then
						needShadow = false;
					end
				end
			end
			if (needIF) then
				showABR = true;
				textABR = "Cast Inner Fire!";
				spellABR = ABR_LOCALE.innerFire;
			elseif (needPWF) then
				showABR = true;
				textABR = "Cast PW:Fortitude!";
				spellABR = ABR_LOCALE.powerFortitude;
			elseif (needSpirit and AnnoyingBuffReminderVariables.spiritOn) then
				showABR = true;
				textABR = "Cast Divine Spirit!";
				spellABR = ABR_LOCALE.divineSpirit;
			elseif (needShadow and AnnoyingBuffReminderVariables.shadowOn) then
				showABR = true;
				textABR = "Cast Shadow Prot!";
				spellABR = ABR_LOCALE.shadowProtection;
			end
		elseif (className == "WARLOCK") then
			-- armor spell, detect, soul link, demon sac, spell/ffirestone weapon enchant
			local needArmor = true;
			local needLink = true;
			local needDetect = true;
			local needDemonSac = true;
			for i = 1, 40 do
				buffName = UnitAura("player", i, "HELPFUL"); -- note not using "PLAYER" filter since Soul Link technically cast by demon
				if (buffName) then
					if (buffName == ABR_LOCALE.demonSkin
						or buffName == ABR_LOCALE.demonArmor
						or buffName == ABR_LOCALE.felArmor) then
							needArmor = false;
					elseif (buffName == ABR_LOCALE.detectInvisibility) then
						needDetect = false;
					elseif (buffName == ABR_LOCALE.soulLink) then
						needLink = false;
					elseif (buffName == ABR_LOCALE.demonicSacrifice1
						or buffName == ABR_LOCALE.demonicSacrifice2
						or buffName == ABR_LOCALE.demonicSacrifice3
						or buffName == ABR_LOCALE.demonicSacrifice4
						or buffName == ABR_LOCALE.demonicSacrifice5) then
							needDemonSac = false;
					end
				end
			end
			if (needArmor) then
				showABR = true;
				textABR = "Cast an Armor!";
				spellABR = ABR_LOCALE.felArmor;
			elseif (needLink and AnnoyingBuffReminderVariables.linkOn) then
				showABR = true;
				textABR = "Cast Soul Link!";
				spellABR = ABR_LOCALE.soulLink;
			elseif (needDetect and AnnoyingBuffReminderVariables.detectOn) then
				showABR = true;
				textABR = "Cast Detect Invis!";
				spellABR = ABR_LOCALE.detectInvisibility;
			elseif (needDemonSac and inCombat and AnnoyingBuffReminderVariables.demonsacOn) then
				showABR = true;
				textABR = "Sacrifice a Demon!";
				spellABR = ABR_LOCALE.demonicSacrifice1;
			elseif (inCombat and AnnoyingBuffReminderVariables.stoneOn) then
				-- mainhand weapon enchant
				if (not GetWeaponEnchantInfo()) then
					showABR = true;
					textABR = "|cff0000ffUse Spell/Firestone!";
					spellABR = ABR_LOCALE.spellstone;
				end
			end
		elseif (className == "MAGE") then
			-- armor spell, ai, amp, damp
			local needArmor = true;
			local needAI = true;
			local needAmp = true;
			local needDamp = true;
			for i = 1, 40 do
				buffName = UnitAura("player", i, "HELPFUL");
				if (buffName) then
					if (buffName == ABR_LOCALE.frostArmor
						or buffName == ABR_LOCALE.iceArmor
						or buffName == ABR_LOCALE.mageArmor
						or buffName == ABR_LOCALE.moltenArmor) then
							needArmor = false;
					elseif (buffName == ABR_LOCALE.arcaneIntellect
						or buffName == ABR_LOCALE.arcaneBrilliance
						or buffName == ABR_LOCALE.dalaranIntellect
						or buffName == ABR_LOCALE.dalaranBrilliance
						or buffName == ABR_LOCALE.songOfVictory) then
							needAI = false;
					elseif (buffName == ABR_LOCALE.amplifyMagic) then
						needAmp = false;
					elseif (buffName == ABR_LOCALE.dampenMagic) then
						needDamp = false;
					end
				end
			end
			if (needArmor) then
				showABR = true;
				textABR = "Cast an Armor!";
				spellABR = ABR_LOCALE.moltenArmor;
			elseif (needAI) then
				showABR = true;
				textABR = "Cast Arcane Int!";
				spellABR = ABR_LOCALE.arcaneIntellect;
			elseif (needAmp and AnnoyingBuffReminderVariables.ampMagicOn) then
				showABR = true;
				textABR = "Cast Amplify Magic!";
				spellABR = ABR_LOCALE.amplifyMagic;
			elseif (needDamp and AnnoyingBuffReminderVariables.dampMagicOn) then
				showABR = true;
				textABR = "Cast Dampen Magic!";
				spellABR = ABR_LOCALE.dampenMagic;
			end
		elseif (className == "SHAMAN") then
			-- mainhand weapon enchant
			local mainhandEnchanted, _, _, offhandEnchanted = GetWeaponEnchantInfo();
			if (not mainhandEnchanted) then
				showABR = true;
				textABR = "|cff0000ffEnchant Mainhand!";
				spellABR = ABR_LOCALE.flametongueWeapon;
			end
			-- offhand weapon enchant
			if (not showABR) then
				local offhandLink = GetInventoryItemLink("player", 17);
				if (offhandLink) then
					local _, _, _, _, _, itemType = GetItemInfo(offhandLink);
					if (itemType == ABR_LOCALE.itemTypeWeapon and not offhandEnchanted) then
						showABR = true;
						textABR = "|cff0000ffEnchant Offhand!";
						spellABR = ABR_LOCALE.flametongueWeapon;
					end
				end
			end
			-- shield spell
			if (not showABR) then
				local needShield = true;
				for i = 1, 40 do
					-- use PLAYER filter and casterID since only need to track *player*-cast spells
					buffName, _, _, _, _, _, _, casterID, _ = UnitAura("player", i, "HELPFUL|PLAYER");
					if (buffName and casterID == "player") then
						if (buffName == ABR_LOCALE.lightningShield
							or buffName == ABR_LOCALE.waterShield
							or buffName == ABR_LOCALE.earthShield) then
								needShield = false;
						end
					end
				end
				if (needShield and not AnnoyingBuffReminderVariables.shamanShieldOff) then
					showABR = true;
					textABR = "Cast a Shield!";
					spellABR = ABR_LOCALE.waterShield;
				end
			end
			-- don't show if in ghost wolf (check this here instead of by "mounted" check due to need to scan buffs)
			if (showABR) then
				for i = 1, 40 do
					-- use PLAYER filter and casterID since only need to track *player*-cast spells
					buffName, _, _, _, _, _, _, casterID, _ = UnitAura("player", i, "HELPFUL|PLAYER");
					if (buffName and casterID == "player") then
						if (buffName == ABR_LOCALE.ghostWolf) then
							AnnoyingBuffReminderHoverFrame:Hide();
							return;
						end
					end
				end
			end
		elseif (className == "HUNTER") then
			-- aspect, tsa
			local needAspect = true;
			local needNonViperAspect = true;
			local needTrueshot = true;
			for i = 1, 40 do
				-- use PLAYER filter and casterID since only need to track *player*-cast spells
				buffName, _, _, _, _, _, _, casterID, _ = UnitAura("player", i, "HELPFUL|PLAYER");
				if (buffName and casterID == "player") then
					if (string.match(buffName, ABR_LOCALE.regexAspect)) then
						needAspect = false;
						if (buffName ~= ABR_LOCALE.aspectOfTheViper) then
							needNonViperAspect = false;
						end
					elseif (buffName == ABR_LOCALE.trueshotAura) then
						needTrueshot = false;
					end
				end
			end
			if (needAspect) then
				showABR = true;
				textABR = "Cast an Aspect!";
				spellABR = ABR_LOCALE.aspectHawk;
			elseif (needNonViperAspect and (UnitPower("player", SPELL_POWER_MANA) * 100 / UnitPowerMax("player", SPELL_POWER_MANA) >= 90)) then
				showABR = true;
				textABR = "Non-Viper Aspect!";
				spellABR = ABR_LOCALE.aspectHawk;
			elseif (needTrueshot and AnnoyingBuffReminderVariables.trueshotOn) then
				showABR = true;
				textABR = "Cast Trueshot Aura!";
				spellABR = ABR_LOCALE.trueshotAura;
			end
		elseif (className == "ROGUE") then
			-- hunger for blood
			local needHunger = true;
			for i = 1, 40 do
				-- use PLAYER filter and casterID since only need to track *player*-cast spells
				buffName, _, _, _, _, _, _, casterID, _ = UnitAura("player", i, "HELPFUL|PLAYER");
				if (buffName and casterID == "player") then
					if (buffName == ABR_LOCALE.hungerForBlood) then
						needHunger = false;
					end
				end
			end
			if (needHunger and inCombat and AnnoyingBuffReminderVariables.hungerOn) then
				showABR = true;
				textABR = "Hunger for Blood!";
				spellABR = ABR_LOCALE.hungerForBlood;
			end
		elseif (className == "PALADIN") then
			-- righteous fury, aura (crusader when mounted, non-crusader when not mounted), blessing
			local needRF = true;
			local needNonCrusaderAura = true;
			local needCrusaderAura = true;
			local needBlessing = true;
			local needSacredShield = true;
			local needSeal = true;
			for i = 1, 40 do
				-- use PLAYER filter and casterID since only need to track *player*-cast spells
				buffName, _, _, _, _, _, _, casterID, _ = UnitAura("player", i, "HELPFUL|PLAYER");
				if (buffName and casterID == "player") then
					if (buffName == ABR_LOCALE.righteousFury) then
						needRF = false;
					elseif (buffName == ABR_LOCALE.crusaderAura) then
						needCrusaderAura = false;
					elseif (string.match(buffName, ABR_LOCALE.regexAura)
						and buffName ~= ABR_LOCALE.crusaderAura) then
							needNonCrusaderAura = false;
					elseif (buffName == ABR_LOCALE.sacredShield) then
						needSacredShield = false;
					elseif (string.match(buffName, ABR_LOCALE.regexBlessing)) then
						needBlessing = false;
					elseif (string.match(buffName, ABR_LOCALE.regexSeal)) then
						needSeal = false;
					end
				end
			end
			if (IsMounted()) then -- check for Crusader if mounted, other buffs otherwise
				if (AnnoyingBuffReminderVariables.crusaderOn and needCrusaderAura and UnitLevel("player") >= 62) then
					showABR = true;
					textABR = "Crusader Aura!";
					spellABR = ABR_LOCALE.crusaderAura;
				end
			else
				if (needRF and AnnoyingBuffReminderVariables.righteousFuryOn) then
					showABR = true;
					textABR = "Cast Righteous Fury!";
					spellABR = ABR_LOCALE.righteousFury;
				elseif (needNonCrusaderAura) then
					showABR = true;
					textABR = "Cast an Aura!";
					spellABR = ABR_LOCALE.devotionAura;
				elseif (needBlessing) then
					showABR = true;
					textABR = "Cast a Blessing!";
					spellABR = ABR_LOCALE.blessingWisdom;
				elseif (needSeal and inCombat and AnnoyingBuffReminderVariables.sealOn) then
					showABR = true;
					textABR = "Cast a Seal!";
					spellABR = ABR_LOCALE.sealRighteousness;
				elseif (needSacredShield and inCombat and AnnoyingBuffReminderVariables.sacredShieldOn) then
					showABR = true;
					textABR = "Cast Sacred Shield!";
					spellABR = ABR_LOCALE.sacredShield;
				end
			end
		elseif (className == "DRUID") then
			-- motw, thorns, omen
			local needMark = true;
			local needThorns = true;
			local needOmen = true;
			for i = 1, 40 do
				buffName = UnitAura("player", i, "HELPFUL");
				if (buffName) then
					if (buffName == ABR_LOCALE.markWild or buffName == ABR_LOCALE.giftWild) then
						needMark = false;
					elseif (buffName == ABR_LOCALE.thorns) then
						needThorns = false;
					elseif (buffName == ABR_LOCALE.omenOfClarity) then
						needOmen = false;
					end
				end
			end
			if (needMark) then
				showABR = true;
				textABR = "Cast Mark of the Wild!";
				spellABR = ABR_LOCALE.markWild;
			elseif (needThorns and AnnoyingBuffReminderVariables.thornsOn) then
				showABR = true;
				textABR = "Cast Thorns!";
				spellABR = ABR_LOCALE.thorns;
			elseif (needOmen and AnnoyingBuffReminderVariables.OmenOn) then
				showABR = true;
				textABR = "Cast Omen of Clarity!";
				spellABR = ABR_LOCALE.omenOfClarity;
			end
		elseif (className == "DEATHKNIGHT") then
			-- presence, horn of winter, bone shield
			local needPresence = true;
			local needHorn = true;
			local needBoneShield = true;
			local needUnholyBlight = true;
			for i = 1, 40 do
				buffName = UnitAura("player", i, "HELPFUL");
				if (buffName) then
					if (buffName == ABR_LOCALE.bloodPresence or buffName == ABR_LOCALE.frostPresence or buffName == ABR_LOCALE.unholyPresence) then
						needPresence = false;
					elseif (buffName == ABR_LOCALE.hornOfWinter or buffName == ABR_LOCALE.strengthOfEarth) then
						needHorn = false;
					elseif (buffName == ABR_LOCALE.boneShield) then
						needBoneShield = false;
					elseif (buffName == ABR_LOCALE.unholyBlight) then
						needUnholyBlight = false;
					end
				end
			end
			if (needPresence) then
				showABR = true;
				textABR = "Cast a Presence!";
				spellABR = ABR_LOCALE.bloodPresence;
			elseif (needHorn and inCombat and AnnoyingBuffReminderVariables.hornOn) then
				showABR = true;
				textABR = "Horn of Winter!";
				spellABR = ABR_LOCALE.hornOfWinter;
			elseif (needBoneShield and inCombat and AnnoyingBuffReminderVariables.boneShieldOn
				and IsUsableSpell(ABR_LOCALE.boneShield) and GetSpellCooldown(ABR_LOCALE.boneShield) == 0) then
				showABR = true;
				textABR = "Cast Bone Shield!";
				spellABR = ABR_LOCALE.boneShield;
			elseif (needUnholyBlight and inCombat and AnnoyingBuffReminderVariables.blightOn and UnitPower("player", SPELL_POWER_RUNIC_POWER) >= 40) then
				showABR = true;
				textABR = "Cast Unholy Blight!";
				spellABR = ABR_LOCALE.unholyBlight;
			end
		end -- end classes ifelseif
		
		-- display or hide ABR as needed
		if (showABR and (
			(UnitPower("player", SPELL_POWER_MANA) * 100 / UnitPowerMax("player", SPELL_POWER_MANA) >= 50) -- if >= 50% mana
			or (className == "ROGUE" and inCombat) -- or a rogue in combat
			or (className == "DEATHKNIGHT" and inCombat) -- or a DK in combat
			or (className == "SHAMAN" and textABR == "Cast a Shield!") -- or a shaman showing a no-mana-cost shield reminder
		)) then
			AnnoyingBuffReminderText:SetText(textABR);
			AnnoyingBuffReminderHoverFrame:Show();
			if (AnnoyingBuffReminderVariables.dontShowIcons) then
				AnnoyingBuffReminderSpellButton1Icon:Hide();
				AnnoyingBuffReminderSpellButton2Icon:Hide();
				AnnoyingBuffReminderHoverFrame:SetWidth(200);
			else
				local texture = select(3, GetSpellInfo(spellABR));
				AnnoyingBuffReminderSpellButton1Icon:SetTexture(texture);
				AnnoyingBuffReminderSpellButton2Icon:SetTexture(texture);
				AnnoyingBuffReminderSpellButton1Icon:Show();
				AnnoyingBuffReminderSpellButton2Icon:Show();
				AnnoyingBuffReminderHoverFrame:SetWidth(265);
			end
		else
			AnnoyingBuffReminderHoverFrame:Hide();
		end
		return;
	end -- end UNIT_MANA/UNIT_AURA if
	
end
